#ifndef _SCDOC_PARSER_H
#define _SCDOC_PARSER_H
#include <stdarg.h>
#include <stdint.h>
#include <stdio.h>

struct parser {
	FILE *input, *output;
	int line, col;
	int qhead;
	uint32_t queue[32];
	uint32_t flags;
	const char *str;
	int fmt_line, fmt_col;
	int p_open;
	int t_open;
	int sh_open;
	int ss_open;
};

enum formatting {
	FORMAT_BOLD = 1,
	FORMAT_UNDERLINE = 2,
	FORMAT_LAST = 4,
};

void parser_fatal(struct parser *parser, const char *err);
uint32_t parser_getch(struct parser *parser);
void parser_pushch(struct parser *parser, uint32_t ch);
void parser_pushstr(struct parser *parser, const char *str);
int start_paragraph(struct parser *p, int indent);
int end_paragraph(struct parser *p);
void start_section(struct parser *p);
void end_section(struct parser *p);
void start_subsection(struct parser *p);
void end_subsection(struct parser *p);
void html_escape(struct parser *p, const char *value);
void html_id(struct parser *p, const char *value);
void html_id_fragment(struct parser *p, const char *value);
void *xcalloc(size_t n, size_t s);
void *xrealloc(void *p, size_t s);

#endif
